#! /usr/bin/env node
import * as Fs from 'fs';
import * as Path from 'path';
import * as Js_exn from 'rescript/lib/es6/js_exn.js';
import * as Belt_Option from 'rescript/lib/es6/belt_Option.js';
import * as Caml_option from 'rescript/lib/es6/caml_option.js';
import * as Js_dict from 'rescript/lib/es6/js_dict.js';
import * as Js_types from 'rescript/lib/es6/js_types.js';
import * as Caml_exceptions from 'rescript/lib/es6/caml_exceptions.js';
import * as Caml_js_exceptions from 'rescript/lib/es6/caml_js_exceptions.js';
import * as Belt_Result from 'rescript/lib/es6/belt_Result.js';
import * as Process from 'process';
import Minimist from 'minimist';
import * as Colorette from 'colorette';

// Generated by ReScript, PLEASE EDIT WITH CARE

function make$9(path, kind) {
  return {
    path: path,
    kind: kind
  };
}
function getLink(lintIssue) {
  var match = lintIssue.kind;
  if (match.TAG === /* InvalidStdlibParentDirName */4) {
    return lintIssue.path;
  } else {
    return "" + lintIssue.path + ":" + match.line.toString() + "";
  }
}
function getMessage(lintIssue) {
  var match = lintIssue.kind;
  switch (match.TAG | 0) {
    case /* ProhibitedModuleOpen */0:
      return "Found \"" + match.prohibitedModuleName + "\" module open.";
    case /* ProhibitedModuleInclude */1:
      return "Found \"" + match.prohibitedModuleName + "\" module include.";
    case /* ProhibitedModuleAssign */2:
    case /* ProhibitedModuleUsage */3:
      return "Found \"" + match.prohibitedModuleName + "\" module usage.";
    case /* InvalidStdlibParentDirName */4:
      return "The \"" + match.stdlibParentDirName + "\" is an invalid directory name for custom standary library related files. Put the files to the directory with the name \"stdlib\" to make it easier to find them.";
  }
}
/* No side effect */

// Generated by ReScript, PLEASE EDIT WITH CARE
function getExnWithMessage(option, message) {
  if (option !== undefined) {
    return Caml_option.valFromOption(option);
  } else {
    return Js_exn.raiseError(message);
  }
}
var forEach = Belt_Option.forEach;
var map$1 = Belt_Option.map;
/* No side effect */

// Generated by ReScript, PLEASE EDIT WITH CARE
var defaultProhibitedModuleNames = ["Belt", "Js", "ReScriptJs"];
function fromBscFlag(bscFlag) {
  if (bscFlag.includes("-open")) {
    return bscFlag.replace("-open", "").trim().split(".")[0];
  }
}
function fromPath(path) {
  var pathWithoutResExtension = path.replace(/\.resi?$/, "");
  if (path.length === pathWithoutResExtension.length) {
    return;
  } else {
    return getExnWithMessage(pathWithoutResExtension.split("/").at(-1), "After splitting path by slashes there should always be at least one item.");
  }
}
function isSubmodule(moduleName, ofModule) {
  return moduleName.startsWith("" + ofModule + "_");
}
var defaultStdlibModuleName = "Stdlib";
/* No side effect */

// Generated by ReScript, PLEASE EDIT WITH CARE

function make$8(param) {
  return [];
}
function addIssue(lintContext, lintIssue) {
  lintContext.push(lintIssue);
}
function getIssues(lintContext) {
  return lintContext;
}
/* No side effect */

// Generated by ReScript, PLEASE EDIT WITH CARE
function make$7(content, path) {
  return {
    content: content,
    path: path,
    moduleName: getExnWithMessage(fromPath(path), "A ResFile should always have a valid module name.")
  };
}
function normalizeName(name) {
  return name.replace(/\W/g, "").toLowerCase();
}
function lint$1(resFile, lintContext, prohibitedModuleNames, stdlibModuleName) {
  if (resFile.moduleName === stdlibModuleName || isSubmodule(resFile.moduleName, stdlibModuleName)) {
    var stdlibParentDirName = getExnWithMessage(resFile.path.split("/").at(-2), "A ResFile should always have a directory name");
    if (normalizeName(stdlibModuleName) !== normalizeName(stdlibParentDirName)) {
      return addIssue(lintContext, make$9(resFile.path, {
        TAG: /* InvalidStdlibParentDirName */4,
        stdlibParentDirName: stdlibParentDirName
      }));
    } else {
      return;
    }
  }
  resFile.content.split("\n").forEach(function (line, idx) {
    prohibitedModuleNames.forEach(function (prohibitedModuleName) {
      var openRe = new RegExp("^ *open " + prohibitedModuleName + "($|\\.)");
      if (openRe.test(line)) {
        addIssue(lintContext, make$9(resFile.path, {
          TAG: /* ProhibitedModuleOpen */0,
          line: idx + 1 | 0,
          prohibitedModuleName: prohibitedModuleName
        }));
      }
      var includeRe = new RegExp("^ *include " + prohibitedModuleName + "($|\\.)");
      if (includeRe.test(line)) {
        addIssue(lintContext, make$9(resFile.path, {
          TAG: /* ProhibitedModuleInclude */1,
          line: idx + 1 | 0,
          prohibitedModuleName: prohibitedModuleName
        }));
      }
      var assignRe = new RegExp("module.+= " + prohibitedModuleName + "($|\\.)");
      if (assignRe.test(line)) {
        addIssue(lintContext, make$9(resFile.path, {
          TAG: /* ProhibitedModuleAssign */2,
          line: idx + 1 | 0,
          prohibitedModuleName: prohibitedModuleName
        }));
      }
      var usageRe = new RegExp("(\\W|^)" + prohibitedModuleName + "\\.");
      if (usageRe.test(line)) {
        return addIssue(lintContext, make$9(resFile.path, {
          TAG: /* ProhibitedModuleUsage */3,
          line: idx + 1 | 0,
          prohibitedModuleName: prohibitedModuleName
        }));
      }
    });
  });
}
function checkIsResFile(dirItem) {
  if (dirItem.endsWith(".res")) {
    return true;
  } else {
    return dirItem.endsWith(".resi");
  }
}
/* No side effect */

// Generated by ReScript, PLEASE EDIT WITH CARE

var Exception = /* @__PURE__ */Caml_exceptions.create("S-ReScriptStruct.Error.Internal.Exception");

function raise(expected, received, initialPathOpt, param) {
  var initialPath = initialPathOpt !== undefined ? initialPathOpt : "";
  throw {
        RE_EXN_ID: Exception,
        _1: {
          c: {
            TAG: /* UnexpectedValue */2,
            expected: expected === undefined ? "undefined" : JSON.stringify(expected),
            received: received === undefined ? "undefined" : JSON.stringify(received)
          },
          p: initialPath
        },
        Error: new Error()
      };
}

function raise$1(code) {
  throw {
        RE_EXN_ID: Exception,
        _1: {
          c: code,
          p: ""
        },
        Error: new Error()
      };
}

function toParseError(internalError) {
  var path = internalError.p;
  var tmp = path === "" ? [] : path.split(",");
  return {
          operation: /* Parsing */1,
          code: internalError.c,
          path: tmp
        };
}

function toSerializeError(internalError) {
  var path = internalError.p;
  var tmp = path === "" ? [] : path.split(",");
  return {
          operation: /* Serializing */0,
          code: internalError.c,
          path: tmp
        };
}

function prependLocation(error, $$location) {
  var path = error.p;
  var tmp = path === "" ? $$location : "" + $$location + "," + path + "";
  return {
          c: error.c,
          p: tmp
        };
}

function panic$1(param) {
  throw new Error("[rescript-struct] Unreachable");
}

function raise$2(message) {
  throw {
        RE_EXN_ID: Exception,
        _1: {
          c: {
            TAG: /* OperationFailed */0,
            _0: message
          },
          p: ""
        },
        Error: new Error()
      };
}

function toReason(nestedLevelOpt, error) {
  var nestedLevel = nestedLevelOpt !== undefined ? nestedLevelOpt : 0;
  var reason = error.code;
  if (typeof reason === "number") {
    switch (reason) {
      case /* MissingParser */0 :
          return "Struct parser is missing";
      case /* MissingSerializer */1 :
          return "Struct serializer is missing";
      case /* UnexpectedAsync */2 :
          return "Encountered unexpected asynchronous transform or refine. Use parseAsyncWith instead of parseWith";
      
    }
  } else {
    switch (reason.TAG | 0) {
      case /* OperationFailed */0 :
          return reason._0;
      case /* UnexpectedType */1 :
      case /* UnexpectedValue */2 :
          break;
      case /* TupleSize */3 :
          return "Expected Tuple with " + reason.expected.toString() + " items, received " + reason.received.toString() + "";
      case /* ExcessField */4 :
          return "Encountered disallowed excess key \"" + reason._0 + "\" on an object. Use Deprecated to ignore a specific field, or S.Object.strip to ignore excess keys completely";
      case /* InvalidUnion */5 :
          var lineBreak = "\n" + " ".repeat((nestedLevel << 1)) + "";
          var array = reason._0.map(function (error) {
                var reason = toReason(nestedLevel + 1, error);
                var nonEmptyPath = error.path;
                var $$location = nonEmptyPath.length !== 0 ? "Failed at " + (
                    nonEmptyPath.length !== 0 ? nonEmptyPath.map(function (pathItem) {
                              return "[" + pathItem + "]";
                            }).join("") : "root"
                  ) + ". " : "";
                return "- " + $$location + "" + reason + "";
              });
          var reasons = Array.from(new Set(array));
          return "Invalid union with following errors" + lineBreak + "" + reasons.join(lineBreak) + "";
      
    }
  }
  return "Expected " + reason.expected + ", received " + reason.received + "";
}

function toString(error) {
  var match = error.operation;
  var operation = match ? "parsing" : "serializing";
  var reason = toReason(undefined, error);
  var path = error.path;
  var pathText = path.length !== 0 ? path.map(function (pathItem) {
            return "[" + pathItem + "]";
          }).join("") : "root";
  return "Failed " + operation + " at " + pathText + ". Reason: " + reason + "";
}

function planSyncTransformation(ctx, transformation) {
  var prevSyncTransformation = ctx.s;
  var prevAsyncTransformation = ctx.a;
  var match = ctx.p;
  if (match !== 1) {
    if (match !== 0) {
      ctx.a = (function (input) {
          return prevAsyncTransformation(input).then(transformation);
        });
    } else {
      ctx.p = /* OnlySync */1;
      ctx.s = transformation;
    }
  } else {
    ctx.s = (function (input) {
        return transformation(prevSyncTransformation(input));
      });
  }
}

function planAsyncTransformation(ctx, transformation) {
  var prevAsyncTransformation = ctx.a;
  var match = ctx.p;
  if (match !== 1) {
    if (match !== 0) {
      ctx.a = (function (input) {
          return prevAsyncTransformation(input).then(transformation);
        });
    } else {
      ctx.p = /* OnlyAsync */2;
      ctx.a = transformation;
    }
  } else {
    ctx.p = /* SyncAndAsync */3;
    ctx.a = transformation;
  }
}

function empty(param, param$1) {
  
}

function compile(transformationFactory, struct) {
  var ctx = {
    p: /* NoTransformation */0,
    s: undefined,
    a: undefined
  };
  transformationFactory(ctx, struct);
  var match = ctx.p;
  switch (match) {
    case /* NoTransformation */0 :
        return /* NoOperation */0;
    case /* OnlySync */1 :
        return {
                TAG: /* SyncOperation */0,
                _0: ctx.s
              };
    case /* OnlyAsync */2 :
        return {
                TAG: /* AsyncOperation */1,
                _0: (function (input) {
                    return function () {
                      return ctx.a(input);
                    };
                  })
              };
    case /* SyncAndAsync */3 :
        return {
                TAG: /* AsyncOperation */1,
                _0: (function (input) {
                    var syncOutput = ctx.s(input);
                    return function () {
                      return ctx.a(syncOutput);
                    };
                  })
              };
    
  }
}

function getParseOperation(struct) {
  var parseOperationState = struct.r;
  if (typeof parseOperationState !== "number") {
    return parseOperationState;
  }
  if (parseOperationState === 2) {
    return {
            TAG: /* SyncOperation */0,
            _0: (function (input) {
                return struct.p(input);
              })
          };
  }
  if (parseOperationState === 3) {
    return {
            TAG: /* AsyncOperation */1,
            _0: (function (input) {
                return struct.a(input);
              })
          };
  }
  struct.r = parseOperationState === 1 ? 3 : 2;
  var compiledParseOperation = compile(struct.pf, struct);
  struct.r = compiledParseOperation;
  return compiledParseOperation;
}

function getSerializeOperation(struct) {
  var serializeOperationState = struct.e;
  if (typeof serializeOperationState !== "number") {
    return serializeOperationState;
  }
  if (serializeOperationState === 1) {
    return (function (input) {
              return struct.s(input);
            });
  }
  struct.e = 1;
  var fn = compile(struct.sf, struct);
  var compiledSerializeOperation;
  compiledSerializeOperation = typeof fn === "number" ? undefined : (
      fn.TAG === /* SyncOperation */0 ? fn._0 : panic$1()
    );
  struct.e = compiledSerializeOperation;
  return compiledSerializeOperation;
}

function raiseUnexpectedTypeError(input, struct) {
  var number = Js_types.classify(input);
  var tmp;
  if (typeof number === "number") {
    switch (number) {
      case /* JSFalse */0 :
      case /* JSTrue */1 :
          tmp = "Bool";
          break;
      case /* JSNull */2 :
          tmp = "Null";
          break;
      case /* JSUndefined */3 :
          tmp = "Option";
          break;
      
    }
  } else {
    switch (number.TAG | 0) {
      case /* JSNumber */0 :
          tmp = Number.isNaN(number._0) ? "NaN Literal (NaN)" : "Float";
          break;
      case /* JSString */1 :
          tmp = "String";
          break;
      case /* JSFunction */2 :
          tmp = "Function";
          break;
      case /* JSObject */3 :
          tmp = Array.isArray(number._0) ? "Array" : "Object";
          break;
      case /* JSSymbol */4 :
          tmp = "Symbol";
          break;
      case /* JSBigInt */5 :
          tmp = "BigInt";
          break;
      
    }
  }
  return raise$1({
              TAG: /* UnexpectedType */1,
              expected: struct.n,
              received: tmp
            });
}

function noOperation(input) {
  return input;
}

function initialSerialize(input) {
  var struct = this;
  var fn = getSerializeOperation(struct);
  var compiledSerialize = fn !== undefined ? fn : noOperation;
  struct.s = compiledSerialize;
  return compiledSerialize(input);
}

function intitialParse(input) {
  var struct = this;
  var fn = getParseOperation(struct);
  var compiledParse;
  compiledParse = typeof fn === "number" ? noOperation : (
      fn.TAG === /* SyncOperation */0 ? fn._0 : raise$1(/* UnexpectedAsync */2)
    );
  struct.p = compiledParse;
  return compiledParse(input);
}

function asyncNoopOperation(input) {
  return function () {
    return Promise.resolve(input);
  };
}

function intitialParseAsync(input) {
  var struct = this;
  var fn = getParseOperation(struct);
  var compiledParseAsync;
  if (typeof fn === "number") {
    compiledParseAsync = asyncNoopOperation;
  } else if (fn.TAG === /* SyncOperation */0) {
    var fn$1 = fn._0;
    compiledParseAsync = (function (input) {
        var syncValue = fn$1(input);
        return function () {
          return Promise.resolve(syncValue);
        };
      });
  } else {
    compiledParseAsync = fn._0;
  }
  struct.a = compiledParseAsync;
  return compiledParseAsync(input);
}

function parseWith(any, struct) {
  try {
    return {
            TAG: /* Ok */0,
            _0: struct.p(any)
          };
  }
  catch (raw_internalError){
    var internalError = Caml_js_exceptions.internalToOCamlException(raw_internalError);
    if (internalError.RE_EXN_ID === Exception) {
      return {
              TAG: /* Error */1,
              _0: toParseError(internalError._1)
            };
    }
    throw internalError;
  }
}

function get(struct, id) {
  var option = struct.m;
  if (option !== undefined) {
    return Js_dict.get(Caml_option.valFromOption(option), id);
  }
  
}

function set(struct, id, metadata) {
  var metadataChange = {};
  var maybeMetadataDict = Caml_option.some(Object.assign({}, struct.m, (metadataChange[id] = metadata, metadataChange)));
  return {
          n: struct.n,
          t: struct.t,
          pf: struct.pf,
          sf: struct.sf,
          r: 0,
          e: 0,
          s: initialSerialize,
          p: intitialParse,
          a: intitialParseAsync,
          i: undefined,
          m: maybeMetadataDict
        };
}

function factory(innerLiteral, variant) {
  var tagged = {
    TAG: /* Literal */0,
    _0: innerLiteral
  };
  var makeParseTransformationFactory = function (literalValue, test) {
    return function (ctx, struct) {
      planSyncTransformation(ctx, (function (input) {
              if (test(input)) {
                if (literalValue === input) {
                  return variant;
                } else {
                  return raise(literalValue, input, undefined);
                }
              } else {
                return raiseUnexpectedTypeError(input, struct);
              }
            }));
    };
  };
  var makeSerializeTransformationFactory = function (output) {
    return function (ctx, param) {
      planSyncTransformation(ctx, (function (input) {
              if (input === variant) {
                return output;
              } else {
                return raise(variant, input, undefined);
              }
            }));
    };
  };
  if (typeof innerLiteral === "number") {
    switch (innerLiteral) {
      case /* EmptyNull */0 :
          var serializeTransformationFactory = makeSerializeTransformationFactory(null);
          return {
                  n: "EmptyNull Literal (null)",
                  t: tagged,
                  pf: (function (ctx, struct) {
                      planSyncTransformation(ctx, (function (input) {
                              if (input === null) {
                                return variant;
                              } else {
                                return raiseUnexpectedTypeError(input, struct);
                              }
                            }));
                    }),
                  sf: serializeTransformationFactory,
                  r: 0,
                  e: 0,
                  s: initialSerialize,
                  p: intitialParse,
                  a: intitialParseAsync,
                  i: undefined,
                  m: undefined
                };
      case /* EmptyOption */1 :
          var serializeTransformationFactory$1 = makeSerializeTransformationFactory(undefined);
          return {
                  n: "EmptyOption Literal (undefined)",
                  t: tagged,
                  pf: (function (ctx, struct) {
                      planSyncTransformation(ctx, (function (input) {
                              if (input === undefined) {
                                return variant;
                              } else {
                                return raiseUnexpectedTypeError(input, struct);
                              }
                            }));
                    }),
                  sf: serializeTransformationFactory$1,
                  r: 0,
                  e: 0,
                  s: initialSerialize,
                  p: intitialParse,
                  a: intitialParseAsync,
                  i: undefined,
                  m: undefined
                };
      case /* NaN */2 :
          var serializeTransformationFactory$2 = makeSerializeTransformationFactory(NaN);
          return {
                  n: "NaN Literal (NaN)",
                  t: tagged,
                  pf: (function (ctx, struct) {
                      planSyncTransformation(ctx, (function (input) {
                              if (Number.isNaN(input)) {
                                return variant;
                              } else {
                                return raiseUnexpectedTypeError(input, struct);
                              }
                            }));
                    }),
                  sf: serializeTransformationFactory$2,
                  r: 0,
                  e: 0,
                  s: initialSerialize,
                  p: intitialParse,
                  a: intitialParseAsync,
                  i: undefined,
                  m: undefined
                };
      
    }
  } else {
    switch (innerLiteral.TAG | 0) {
      case /* String */0 :
          var string = innerLiteral._0;
          var serializeTransformationFactory$3 = makeSerializeTransformationFactory(string);
          var parseTransformationFactory = makeParseTransformationFactory(string, (function (input) {
                  return typeof input === "string";
                }));
          return {
                  n: "String Literal (\"" + string + "\")",
                  t: tagged,
                  pf: parseTransformationFactory,
                  sf: serializeTransformationFactory$3,
                  r: 0,
                  e: 0,
                  s: initialSerialize,
                  p: intitialParse,
                  a: intitialParseAsync,
                  i: undefined,
                  m: undefined
                };
      case /* Int */1 :
          var $$int = innerLiteral._0;
          var serializeTransformationFactory$4 = makeSerializeTransformationFactory($$int);
          var parseTransformationFactory$1 = makeParseTransformationFactory($$int, (function (input) {
                  if (typeof input === "number" && input < 2147483648 && input > -2147483649) {
                    return input % 1 === 0;
                  } else {
                    return false;
                  }
                }));
          var name = "Int Literal (" + $$int.toString() + ")";
          return {
                  n: name,
                  t: tagged,
                  pf: parseTransformationFactory$1,
                  sf: serializeTransformationFactory$4,
                  r: 0,
                  e: 0,
                  s: initialSerialize,
                  p: intitialParse,
                  a: intitialParseAsync,
                  i: undefined,
                  m: undefined
                };
      case /* Float */2 :
          var $$float = innerLiteral._0;
          var serializeTransformationFactory$5 = makeSerializeTransformationFactory($$float);
          var parseTransformationFactory$2 = makeParseTransformationFactory($$float, (function (input) {
                  return typeof input === "number";
                }));
          var name$1 = "Float Literal (" + $$float.toString() + ")";
          return {
                  n: name$1,
                  t: tagged,
                  pf: parseTransformationFactory$2,
                  sf: serializeTransformationFactory$5,
                  r: 0,
                  e: 0,
                  s: initialSerialize,
                  p: intitialParse,
                  a: intitialParseAsync,
                  i: undefined,
                  m: undefined
                };
      case /* Bool */3 :
          var bool = innerLiteral._0;
          var serializeTransformationFactory$6 = makeSerializeTransformationFactory(bool);
          var parseTransformationFactory$3 = makeParseTransformationFactory(bool, (function (input) {
                  return typeof input === "boolean";
                }));
          var name$2 = "Bool Literal (" + bool.toString() + ")";
          return {
                  n: name$2,
                  t: tagged,
                  pf: parseTransformationFactory$3,
                  sf: serializeTransformationFactory$6,
                  r: 0,
                  e: 0,
                  s: initialSerialize,
                  p: intitialParse,
                  a: intitialParseAsync,
                  i: undefined,
                  m: undefined
                };
      
    }
  }
}

function factory$1(innerLiteral) {
  if (typeof innerLiteral === "number") {
    return factory(innerLiteral, undefined);
  } else {
    return factory(innerLiteral, innerLiteral._0);
  }
}

var metadataId = "rescript-struct:Object_UnknownKeys";

function classify$1(struct) {
  var option = get(struct, metadataId);
  if (option !== undefined) {
    return Caml_option.valFromOption(option);
  } else {
    return /* Strip */1;
  }
}

var value = Symbol("rescript-struct:Object.FieldDefinition");

function analyzeDefinition(definition, definerCtx, path, inlinedPath) {
  if (definition === value) {
    var originalFieldName = definerCtx.originalFieldNames[definerCtx.registeredFieldsCount];
    definerCtx.registeredFieldsCount = definerCtx.registeredFieldsCount + 1;
    definerCtx.definedFieldInstructions.push({
          TAG: /* Registered */1,
          fieldStruct: definerCtx.originalFields[originalFieldName],
          originalFieldName: originalFieldName,
          inlinedOriginalFieldName: JSON.stringify(originalFieldName),
          inlinedPath: inlinedPath,
          path: path
        });
    return ;
  }
  if (typeof definition === "object" && definition !== null) {
    definerCtx.inlinedPreparationPathes.push(inlinedPath);
    definerCtx.inlinedPreparationValues.push(Array.isArray(definition) ? "[]" : "{}");
    var definitionFieldNames = Object.keys(definition);
    for(var idx = 0 ,idx_finish = definitionFieldNames.length; idx < idx_finish; ++idx){
      var definitionFieldName = definitionFieldNames[idx];
      var fieldDefinition = definition[definitionFieldName];
      var tmp = path === "" ? definitionFieldName : "" + path + "," + definitionFieldName + "";
      analyzeDefinition(fieldDefinition, definerCtx, tmp, "" + inlinedPath + "[" + JSON.stringify(definitionFieldName) + "]");
    }
    return ;
  }
  definerCtx.constantInstructions.push({
        inlinedPath: inlinedPath,
        v: definition,
        path: path
      });
}

function structToInlinedValue(_struct) {
  while(true) {
    var struct = _struct;
    var unionStructs = struct.t;
    if (typeof unionStructs === "number") {
      throw undefined;
    }
    switch (unionStructs.TAG | 0) {
      case /* Literal */0 :
          var string = unionStructs._0;
          if (typeof string !== "number") {
            if (string.TAG === /* String */0) {
              return JSON.stringify(string._0);
            } else {
              return string._0.toString();
            }
          }
          switch (string) {
            case /* EmptyNull */0 :
                return "null";
            case /* EmptyOption */1 :
                return "undefined";
            case /* NaN */2 :
                return "NaN";
            
          }
      case /* Object */4 :
          var fields = unionStructs.fields;
          return "{" + unionStructs.fieldNames.map((function(fields){
                      return function (fieldName) {
                        return "" + JSON.stringify(fieldName) + ":" + structToInlinedValue(fields[fieldName]) + "";
                      }
                      }(fields))).join(",") + "}";
      case /* Tuple */5 :
          return "[" + unionStructs._0.map(structToInlinedValue).join(",") + "]";
      case /* Union */6 :
          _struct = unionStructs._0[0];
          continue ;
      default:
        throw undefined;
    }
  }}

function factory$2(definer) {
  var definerCtx = {
    originalFieldNames: undefined,
    originalFields: {},
    registeredFieldsCount: 0,
    inlinedPreparationPathes: [],
    inlinedPreparationValues: [],
    definedFieldInstructions: [],
    constantInstructions: []
  };
  var definition = definer(definerCtx);
  var originalFieldNames = Object.keys(definerCtx.originalFields);
  definerCtx.originalFieldNames = originalFieldNames;
  analyzeDefinition(definition, definerCtx, "", "");
  var originalFieldNamesCount = originalFieldNames.length;
  if (definerCtx.registeredFieldsCount > originalFieldNamesCount) {
    throw new Error("[rescript-struct] The object defention has more registered fields than expected.");
  }
  if (definerCtx.registeredFieldsCount < originalFieldNamesCount) {
    throw new Error("[rescript-struct] The object defention contains fields that weren't registered.");
  }
  var serializeTransformationFactory = function (ctx, param) {
    var inliningOriginalFieldNameRef = undefined;
    try {
      var constantInstructions = definerCtx.constantInstructions;
      var definedFieldInstructions = definerCtx.definedFieldInstructions;
      var serializeFnsByInstructionIdx = {};
      var stringRef = "";
      for(var idx = 0 ,idx_finish = constantInstructions.length; idx < idx_finish; ++idx){
        var match = constantInstructions[idx];
        var inlinedPath = match.inlinedPath;
        var content = "r(" + idx.toString() + ",t" + inlinedPath + ")";
        var condition = "t" + inlinedPath + "!==d[" + idx.toString() + "].v";
        stringRef = stringRef + ("if(" + condition + "){" + content + "}");
      }
      var constants = stringRef;
      var contentRef = "var i;return{";
      for(var idx$1 = 0 ,idx_finish$1 = definedFieldInstructions.length; idx$1 < idx_finish$1; ++idx$1){
        var definedFieldInstruction = definedFieldInstructions[idx$1];
        var fieldStruct = definedFieldInstruction.fieldStruct;
        var inlinedOriginalFieldName;
        inlinedOriginalFieldName = definedFieldInstruction.TAG === /* Discriminant */0 ? definedFieldInstruction.inlinedOriginalFieldName : definedFieldInstruction.inlinedOriginalFieldName;
        var inlinedInstructionIdx = idx$1.toString();
        var tmp;
        if (definedFieldInstruction.TAG === /* Discriminant */0) {
          var tmp$1;
          tmp$1 = definedFieldInstruction.TAG === /* Discriminant */0 ? definedFieldInstruction.originalFieldName : definedFieldInstruction.originalFieldName;
          inliningOriginalFieldNameRef = tmp$1;
          tmp = "" + inlinedOriginalFieldName + ":" + structToInlinedValue(fieldStruct) + ",";
        } else {
          var inlinedPath$1 = definedFieldInstruction.inlinedPath;
          var fn = getSerializeOperation(fieldStruct);
          if (fn !== undefined) {
            serializeFnsByInstructionIdx[inlinedInstructionIdx] = fn;
            tmp = "" + inlinedOriginalFieldName + ":(i=" + inlinedInstructionIdx + ",s[" + inlinedInstructionIdx + "](t" + inlinedPath$1 + ")),";
          } else {
            tmp = "" + inlinedOriginalFieldName + ":t" + inlinedPath$1 + ",";
          }
        }
        contentRef = contentRef + tmp;
      }
      var tryContent = contentRef + "}";
      var originalObjectConstructionAndReturn = "try{" + tryContent + "}catch(e){c(e,i)}";
      var inlinedSerializeFunction = "function(t){" + ("" + constants + "" + originalObjectConstructionAndReturn + "") + "}";
      planSyncTransformation(ctx, new Function("s", "d", "r", "c", "return " + inlinedSerializeFunction + "")(serializeFnsByInstructionIdx, constantInstructions, (function (instructionIdx, received) {
                  var match = constantInstructions[instructionIdx];
                  return raise(match.v, received, match.path, undefined);
                }), (function (exn, instructionIdx) {
                  var tmp;
                  if (exn.RE_EXN_ID === Exception) {
                    var definedFieldInstruction = definedFieldInstructions[instructionIdx];
                    tmp = definedFieldInstruction.TAG === /* Discriminant */0 ? panic$1(undefined) : ({
                          RE_EXN_ID: Exception,
                          _1: prependLocation(exn._1, definedFieldInstruction.path)
                        });
                  } else {
                    tmp = exn;
                  }
                  throw tmp;
                })));
    }
    catch (exn){
      var inliningOriginalFieldName = inliningOriginalFieldNameRef;
      planSyncTransformation(ctx, (function (param) {
              throw {
                    RE_EXN_ID: Exception,
                    _1: {
                      c: /* MissingSerializer */1,
                      p: inliningOriginalFieldName
                    },
                    Error: new Error()
                  };
            }));
    }
  };
  var parseTransformationFactory = function (ctx, struct) {
    var constantInstructions = definerCtx.constantInstructions;
    var definedFieldInstructions = definerCtx.definedFieldInstructions;
    var inlinedPreparationValues = definerCtx.inlinedPreparationValues;
    var inlinedPreparationPathes = definerCtx.inlinedPreparationPathes;
    var withUnknownKeysRefinement = classify$1(struct) === /* Strict */0;
    var definedAsyncFieldInstructions = [];
    var parseFnsByInstructionIdx = {};
    var refinement = "if(typeof o!==\"object\"||o===null||Array.isArray(o)){u(o)}";
    var stringRef = "var t;";
    for(var idx = 0 ,idx_finish = inlinedPreparationPathes.length; idx < idx_finish; ++idx){
      var preparationPath = inlinedPreparationPathes[idx];
      var preparationInlinedValue = inlinedPreparationValues[idx];
      stringRef = stringRef + ("t" + preparationPath + "=" + preparationInlinedValue + ";");
    }
    var preparation = stringRef;
    var transformedObjectConstruction;
    if (definedFieldInstructions.length === 0) {
      transformedObjectConstruction = "";
    } else {
      var stringRef$1 = "";
      for(var idx$1 = 0 ,idx_finish$1 = definedFieldInstructions.length; idx$1 < idx_finish$1; ++idx$1){
        var definedFieldInstruction = definedFieldInstructions[idx$1];
        var fieldStruct = definedFieldInstruction.fieldStruct;
        var inlinedOriginalFieldName;
        inlinedOriginalFieldName = definedFieldInstruction.TAG === /* Discriminant */0 ? definedFieldInstruction.inlinedOriginalFieldName : definedFieldInstruction.inlinedOriginalFieldName;
        var inlinedInstructionIdx = idx$1.toString();
        var parseOperation = getParseOperation(fieldStruct);
        var maybeParseFn;
        maybeParseFn = typeof parseOperation === "number" ? undefined : parseOperation._0;
        var isAsync;
        isAsync = typeof parseOperation === "number" || parseOperation.TAG === /* SyncOperation */0 ? false : true;
        var inlinedInputData = "o[" + inlinedOriginalFieldName + "]";
        var maybeInlinedDestination;
        if (isAsync) {
          var inlinedDestination = "a[" + definedAsyncFieldInstructions.length.toString() + "]";
          if (definedAsyncFieldInstructions.length === 0) {
            stringRef$1 = stringRef$1 + "var a={};";
          }
          if (definedFieldInstruction.TAG !== /* Discriminant */0) {
            stringRef$1 = stringRef$1 + ("t" + definedFieldInstruction.inlinedPath + "=undefined;");
          }
          definedAsyncFieldInstructions.push(definedFieldInstruction);
          maybeInlinedDestination = inlinedDestination;
        } else {
          maybeInlinedDestination = definedFieldInstruction.TAG === /* Discriminant */0 ? undefined : "t" + definedFieldInstruction.inlinedPath + "";
        }
        var match = fieldStruct.i;
        stringRef$1 = stringRef$1 + (
          maybeParseFn !== undefined ? (
              match !== undefined ? "var v=" + inlinedInputData + ";if(" + match + "){" + (
                  maybeInlinedDestination !== undefined ? "" + maybeInlinedDestination + "=v" : ""
                ) + "}else{i=" + inlinedInstructionIdx + ";s(v,f[" + inlinedOriginalFieldName + "])}" : (parseFnsByInstructionIdx[inlinedInstructionIdx] = maybeParseFn, "i=" + inlinedInstructionIdx + ";" + (
                    maybeInlinedDestination !== undefined ? "" + maybeInlinedDestination + "=" : ""
                  ) + "p[" + inlinedInstructionIdx + "](" + inlinedInputData + ");")
            ) : (
              maybeInlinedDestination !== undefined ? "" + maybeInlinedDestination + "=" + inlinedInputData + ";" : ""
            )
        );
      }
      var tryContent = stringRef$1;
      transformedObjectConstruction = "var i;" + ("try{" + tryContent + "}catch(e){c(e,i)}");
    }
    var unknownKeysRefinement;
    if (withUnknownKeysRefinement) {
      var stringRef$2 = "for(var k in o){switch(k){";
      for(var idx$2 = 0 ,idx_finish$2 = definedFieldInstructions.length; idx$2 < idx_finish$2; ++idx$2){
        var definedFieldInstruction$1 = definedFieldInstructions[idx$2];
        var inlinedOriginalFieldName$1;
        inlinedOriginalFieldName$1 = definedFieldInstruction$1.TAG === /* Discriminant */0 ? definedFieldInstruction$1.inlinedOriginalFieldName : definedFieldInstruction$1.inlinedOriginalFieldName;
        stringRef$2 = stringRef$2 + ("case" + inlinedOriginalFieldName$1 + ":continue;");
      }
      unknownKeysRefinement = stringRef$2 + "default:x(k)}}";
    } else {
      unknownKeysRefinement = "";
    }
    var stringRef$3 = "";
    for(var idx$3 = 0 ,idx_finish$3 = constantInstructions.length; idx$3 < idx_finish$3; ++idx$3){
      var match$1 = constantInstructions[idx$3];
      stringRef$3 = stringRef$3 + ("t" + match$1.inlinedPath + "=d[" + idx$3.toString() + "].v;");
    }
    var constants = stringRef$3;
    var returnValue = definedAsyncFieldInstructions.length === 0 ? "t" : "a.t=t,a";
    var inlinedParseFunction = "function(o){" + ("" + refinement + "" + preparation + "" + transformedObjectConstruction + "" + unknownKeysRefinement + "" + constants + "return " + returnValue + "") + "}";
    planSyncTransformation(ctx, new Function("c", "p", "f", "d", "u", "s", "x", "return " + inlinedParseFunction + "")((function (exn, instructionIdx) {
                var tmp;
                if (exn.RE_EXN_ID === Exception) {
                  var definedFieldInstruction = definedFieldInstructions[instructionIdx];
                  var tmp$1;
                  tmp$1 = definedFieldInstruction.TAG === /* Discriminant */0 ? definedFieldInstruction.originalFieldName : definedFieldInstruction.originalFieldName;
                  tmp = {
                    RE_EXN_ID: Exception,
                    _1: prependLocation(exn._1, tmp$1)
                  };
                } else {
                  tmp = exn;
                }
                throw tmp;
              }), parseFnsByInstructionIdx, definerCtx.originalFields, constantInstructions, (function (input) {
                return raiseUnexpectedTypeError(input, struct);
              }), raiseUnexpectedTypeError, (function (exccessFieldName) {
                return raise$1({
                            TAG: /* ExcessField */4,
                            _0: exccessFieldName
                          });
              })));
    if (definedAsyncFieldInstructions.length <= 0) {
      return ;
    }
    var resolveVar = "rs";
    var rejectVar = "rj";
    var contentRef = "var y=" + definedAsyncFieldInstructions.length.toString() + ",t=a.t;";
    for(var idx$4 = 0 ,idx_finish$4 = definedAsyncFieldInstructions.length; idx$4 < idx_finish$4; ++idx$4){
      var definedAsyncFieldInstruction = definedAsyncFieldInstructions[idx$4];
      var inlinedIdx = idx$4.toString();
      var fieldValueVar = "z";
      var inlinedFieldValueAssignment;
      inlinedFieldValueAssignment = definedAsyncFieldInstruction.TAG === /* Discriminant */0 ? "" : "t" + definedAsyncFieldInstruction.inlinedPath + "=" + fieldValueVar + "";
      var inlinedIteration = "if(y--===1){" + ("" + resolveVar + "(t)") + "}";
      var onFieldSuccessInlinedFnContent = "" + inlinedFieldValueAssignment + ";" + inlinedIteration + "";
      var onFieldSuccessInlinedFn = "function(" + fieldValueVar + "){" + onFieldSuccessInlinedFnContent + "}";
      var errorVar = "z";
      var onFieldErrorInlinedFn = "function(" + errorVar + "){" + ("" + rejectVar + "(j(" + errorVar + "," + inlinedIdx + "))") + "}";
      contentRef = contentRef + ("a[" + inlinedIdx + "]().then(" + onFieldSuccessInlinedFn + "," + onFieldErrorInlinedFn + ");");
    }
    var content = contentRef;
    var inlinedAsyncParseFunction = "function(a){" + ("return " + ("new Promise(function(" + resolveVar + "," + rejectVar + "){" + content + "})") + "") + "}";
    planAsyncTransformation(ctx, new Function("j", "return " + inlinedAsyncParseFunction + "")(function (exn, asyncInstructionIdx) {
              if (exn.RE_EXN_ID !== Exception) {
                return exn;
              }
              var definedFieldInstruction = definedAsyncFieldInstructions[asyncInstructionIdx];
              var tmp;
              tmp = definedFieldInstruction.TAG === /* Discriminant */0 ? definedFieldInstruction.originalFieldName : definedFieldInstruction.originalFieldName;
              return {
                      RE_EXN_ID: Exception,
                      _1: prependLocation(exn._1, tmp)
                    };
            }));
  };
  return {
          n: "Object",
          t: {
            TAG: /* Object */4,
            fields: definerCtx.originalFields,
            fieldNames: definerCtx.originalFieldNames
          },
          pf: parseTransformationFactory,
          sf: serializeTransformationFactory,
          r: 0,
          e: 0,
          s: initialSerialize,
          p: intitialParse,
          a: intitialParseAsync,
          i: undefined,
          m: undefined
        };
}

function field(definerCtx, originalFieldName, struct) {
  definerCtx.originalFields[originalFieldName] = struct;
  return value;
}

function discriminant(definerCtx, originalFieldName, struct) {
  definerCtx.originalFields[originalFieldName] = struct;
  definerCtx.registeredFieldsCount = definerCtx.registeredFieldsCount + 1;
  definerCtx.definedFieldInstructions.unshift({
        TAG: /* Discriminant */0,
        fieldStruct: struct,
        inlinedOriginalFieldName: JSON.stringify(originalFieldName),
        originalFieldName: originalFieldName
      });
}

function strip(struct) {
  return set(struct, metadataId, /* Strip */1);
}

function strict(struct) {
  return set(struct, metadataId, /* Strict */0);
}

function factory$5(param) {
  return {
          n: "String",
          t: /* String */2,
          pf: (function (ctx, struct) {
              planSyncTransformation(ctx, (function (input) {
                      if (typeof input === "string") {
                        return input;
                      } else {
                        return raiseUnexpectedTypeError(input, struct);
                      }
                    }));
            }),
          sf: empty,
          r: 0,
          e: 0,
          s: initialSerialize,
          p: intitialParse,
          a: intitialParseAsync,
          i: "typeof v===\"string\"",
          m: undefined
        };
}

function factory$6(innerStruct) {
  return {
          n: "Json",
          t: /* String */2,
          pf: (function (ctx, struct) {
              var fn = getParseOperation(innerStruct);
              var $$process = typeof fn === "number" ? (function (prim) {
                    return prim;
                  }) : fn._0;
              planSyncTransformation(ctx, (function (input) {
                      if (typeof input !== "string") {
                        return raiseUnexpectedTypeError(input, struct);
                      }
                      var __x;
                      try {
                        __x = JSON.parse(input);
                      }
                      catch (raw_obj){
                        var obj = Caml_js_exceptions.internalToOCamlException(raw_obj);
                        if (obj.RE_EXN_ID === Js_exn.$$Error) {
                          var option = obj._1.message;
                          __x = raise$2(option !== undefined ? Caml_option.valFromOption(option) : "Failed to parse JSON");
                        } else {
                          throw obj;
                        }
                      }
                      return $$process(__x);
                    }));
              var match = getParseOperation(innerStruct);
              if (typeof match === "number" || match.TAG === /* SyncOperation */0) {
                return ;
              } else {
                return planAsyncTransformation(ctx, (function (asyncFn) {
                              return asyncFn();
                            }));
              }
            }),
          sf: (function (ctx, param) {
              var fn = getSerializeOperation(innerStruct);
              if (fn !== undefined) {
                return planSyncTransformation(ctx, (function (input) {
                              return JSON.stringify(fn(input));
                            }));
              } else {
                return planSyncTransformation(ctx, (function (input) {
                              return JSON.stringify(input);
                            }));
              }
            }),
          r: 0,
          e: 0,
          s: initialSerialize,
          p: intitialParse,
          a: intitialParseAsync,
          i: undefined,
          m: undefined
        };
}

function factory$11(innerStruct) {
  return {
          n: "Option",
          t: {
            TAG: /* Option */1,
            _0: innerStruct
          },
          pf: (function (ctx, param) {
              var planSyncTransformation$1 = function (fn) {
                planSyncTransformation(ctx, (function (input) {
                        if (input !== undefined) {
                          return Caml_option.some(fn(Caml_option.valFromOption(input)));
                        }
                        
                      }));
              };
              var fn = getParseOperation(innerStruct);
              if (typeof fn === "number") {
                return ;
              }
              if (fn.TAG === /* SyncOperation */0) {
                return planSyncTransformation$1(fn._0);
              }
              planSyncTransformation$1(fn._0);
              planAsyncTransformation(ctx, (function (input) {
                      if (input !== undefined) {
                        return input().then(function (value) {
                                    return Caml_option.some(value);
                                  });
                      } else {
                        return Promise.resolve(undefined);
                      }
                    }));
            }),
          sf: (function (ctx, param) {
              var fn = getSerializeOperation(innerStruct);
              if (fn !== undefined) {
                return planSyncTransformation(ctx, (function (input) {
                              if (input !== undefined) {
                                return fn(Caml_option.valFromOption(input));
                              }
                              
                            }));
              } else {
                return planSyncTransformation(ctx, (function (input) {
                              if (input !== undefined) {
                                return Caml_option.valFromOption(input);
                              }
                              
                            }));
              }
            }),
          r: 0,
          e: 0,
          s: initialSerialize,
          p: intitialParse,
          a: intitialParseAsync,
          i: undefined,
          m: undefined
        };
}

function factory$13(innerStruct) {
  return {
          n: "Array",
          t: {
            TAG: /* Array */3,
            _0: innerStruct
          },
          pf: (function (ctx, struct) {
              planSyncTransformation(ctx, (function (input) {
                      if (Array.isArray(input) === false) {
                        return raiseUnexpectedTypeError(input, struct);
                      } else {
                        return input;
                      }
                    }));
              var planSyncTransformation$1 = function (fn) {
                planSyncTransformation(ctx, (function (input) {
                        var newArray = [];
                        for(var idx = 0 ,idx_finish = input.length; idx < idx_finish; ++idx){
                          var innerData = input[idx];
                          try {
                            var value = fn(innerData);
                            newArray.push(value);
                          }
                          catch (raw_internalError){
                            var internalError = Caml_js_exceptions.internalToOCamlException(raw_internalError);
                            if (internalError.RE_EXN_ID === Exception) {
                              throw {
                                    RE_EXN_ID: Exception,
                                    _1: prependLocation(internalError._1, idx.toString()),
                                    Error: new Error()
                                  };
                            }
                            throw internalError;
                          }
                        }
                        return newArray;
                      }));
              };
              var fn = getParseOperation(innerStruct);
              if (typeof fn === "number") {
                return ;
              }
              if (fn.TAG === /* SyncOperation */0) {
                return planSyncTransformation$1(fn._0);
              }
              planSyncTransformation$1(fn._0);
              planAsyncTransformation(ctx, (function (input) {
                      return Promise.all(input.map(function (asyncFn, idx) {
                                      return asyncFn().catch(function (exn) {
                                                  throw exn.RE_EXN_ID === Exception ? ({
                                                            RE_EXN_ID: Exception,
                                                            _1: prependLocation(exn._1, idx.toString())
                                                          }) : exn;
                                                });
                                    }));
                    }));
            }),
          sf: (function (ctx, param) {
              var fn = getSerializeOperation(innerStruct);
              if (fn !== undefined) {
                return planSyncTransformation(ctx, (function (input) {
                              var newArray = [];
                              for(var idx = 0 ,idx_finish = input.length; idx < idx_finish; ++idx){
                                var innerData = input[idx];
                                try {
                                  var value = fn(innerData);
                                  newArray.push(value);
                                }
                                catch (raw_internalError){
                                  var internalError = Caml_js_exceptions.internalToOCamlException(raw_internalError);
                                  if (internalError.RE_EXN_ID === Exception) {
                                    throw {
                                          RE_EXN_ID: Exception,
                                          _1: prependLocation(internalError._1, idx.toString()),
                                          Error: new Error()
                                        };
                                  }
                                  throw internalError;
                                }
                              }
                              return newArray;
                            }));
              }
              
            }),
          r: 0,
          e: 0,
          s: initialSerialize,
          p: intitialParse,
          a: intitialParseAsync,
          i: undefined,
          m: undefined
        };
}

var metadataId$2 = "rescript-struct:Defaulted";

function factory$15(innerStruct, defaultValue) {
  return set({
              n: innerStruct.n,
              t: innerStruct.t,
              pf: (function (ctx, param) {
                  var fn = getParseOperation(innerStruct);
                  if (typeof fn === "number") {
                    return planSyncTransformation(ctx, (function (input) {
                                  if (input !== undefined) {
                                    return Caml_option.valFromOption(input);
                                  } else {
                                    return defaultValue;
                                  }
                                }));
                  }
                  if (fn.TAG === /* SyncOperation */0) {
                    var fn$1 = fn._0;
                    return planSyncTransformation(ctx, (function (input) {
                                  var option = fn$1(input);
                                  if (option !== undefined) {
                                    return Caml_option.valFromOption(option);
                                  } else {
                                    return defaultValue;
                                  }
                                }));
                  }
                  planSyncTransformation(ctx, fn._0);
                  planAsyncTransformation(ctx, (function (asyncFn) {
                          return asyncFn().then(function (value) {
                                      if (value !== undefined) {
                                        return Caml_option.valFromOption(value);
                                      } else {
                                        return defaultValue;
                                      }
                                    });
                        }));
                }),
              sf: (function (ctx, param) {
                  var fn = getSerializeOperation(innerStruct);
                  if (fn !== undefined) {
                    return planSyncTransformation(ctx, (function (input) {
                                  var value = Caml_option.some(input);
                                  return fn(value);
                                }));
                  } else {
                    return planSyncTransformation(ctx, (function (input) {
                                  return Caml_option.some(input);
                                }));
                  }
                }),
              r: 0,
              e: 0,
              s: initialSerialize,
              p: intitialParse,
              a: intitialParseAsync,
              i: undefined,
              m: undefined
            }, metadataId$2, /* WithDefaultValue */{
              _0: defaultValue
            });
}

function factory$16(param) {
  var structs = (Array.from(arguments));
  var numberOfStructs = structs.length;
  return {
          n: "Tuple",
          t: {
            TAG: /* Tuple */5,
            _0: structs
          },
          pf: (function (ctx, struct) {
              var noopOps = [];
              var syncOps = [];
              var asyncOps = [];
              for(var idx = 0 ,idx_finish = structs.length; idx < idx_finish; ++idx){
                var innerStruct = structs[idx];
                var fn = getParseOperation(innerStruct);
                if (typeof fn === "number") {
                  noopOps.push(idx);
                } else if (fn.TAG === /* SyncOperation */0) {
                  syncOps.push([
                        idx,
                        fn._0
                      ]);
                } else {
                  syncOps.push([
                        idx,
                        fn._0
                      ]);
                  asyncOps.push(idx);
                }
              }
              var withAsyncOps = asyncOps.length > 0;
              planSyncTransformation(ctx, (function (input) {
                      if (Array.isArray(input)) {
                        var numberOfInputItems = input.length;
                        if (numberOfStructs !== numberOfInputItems) {
                          raise$1({
                                TAG: /* TupleSize */3,
                                expected: numberOfStructs,
                                received: numberOfInputItems
                              });
                        }
                        
                      } else {
                        raiseUnexpectedTypeError(input, struct);
                      }
                      var newArray = [];
                      for(var idx = 0 ,idx_finish = syncOps.length; idx < idx_finish; ++idx){
                        var match = syncOps[idx];
                        var originalIdx = match[0];
                        var innerData = input[originalIdx];
                        try {
                          var value = match[1](innerData);
                          newArray[originalIdx] = value;
                        }
                        catch (raw_internalError){
                          var internalError = Caml_js_exceptions.internalToOCamlException(raw_internalError);
                          if (internalError.RE_EXN_ID === Exception) {
                            throw {
                                  RE_EXN_ID: Exception,
                                  _1: prependLocation(internalError._1, idx.toString()),
                                  Error: new Error()
                                };
                          }
                          throw internalError;
                        }
                      }
                      for(var idx$1 = 0 ,idx_finish$1 = noopOps.length; idx$1 < idx_finish$1; ++idx$1){
                        var originalIdx$1 = noopOps[idx$1];
                        var innerData$1 = input[originalIdx$1];
                        newArray[originalIdx$1] = innerData$1;
                      }
                      if (withAsyncOps) {
                        return newArray;
                      } else if (numberOfStructs !== 0) {
                        if (numberOfStructs !== 1) {
                          return newArray;
                        } else {
                          return newArray[0];
                        }
                      } else {
                        return ;
                      }
                    }));
              if (withAsyncOps) {
                return planAsyncTransformation(ctx, (function (tempArray) {
                              return Promise.all(asyncOps.map(function (originalIdx) {
                                                return tempArray[originalIdx]().catch(function (exn) {
                                                            throw exn.RE_EXN_ID === Exception ? ({
                                                                      RE_EXN_ID: Exception,
                                                                      _1: prependLocation(exn._1, originalIdx.toString())
                                                                    }) : exn;
                                                          });
                                              })).then(function (values) {
                                          values.forEach(function (value, idx) {
                                                var originalIdx = asyncOps[idx];
                                                tempArray[originalIdx] = value;
                                              });
                                          if (tempArray.length <= 1) {
                                            return tempArray[0];
                                          } else {
                                            return tempArray;
                                          }
                                        });
                            }));
              }
              
            }),
          sf: (function (ctx, param) {
              var serializeOperations = [];
              for(var idx = 0 ,idx_finish = structs.length; idx < idx_finish; ++idx){
                serializeOperations.push(getSerializeOperation(structs[idx]));
              }
              planSyncTransformation(ctx, (function (input) {
                      var inputArray = numberOfStructs === 1 ? [input] : input;
                      var newArray = [];
                      for(var idx = 0 ,idx_finish = serializeOperations.length; idx < idx_finish; ++idx){
                        var innerData = inputArray[idx];
                        var serializeOperation = serializeOperations[idx];
                        if (serializeOperation !== undefined) {
                          try {
                            var value = serializeOperation(innerData);
                            newArray.push(value);
                          }
                          catch (raw_internalError){
                            var internalError = Caml_js_exceptions.internalToOCamlException(raw_internalError);
                            if (internalError.RE_EXN_ID === Exception) {
                              throw {
                                    RE_EXN_ID: Exception,
                                    _1: prependLocation(internalError._1, idx.toString()),
                                    Error: new Error()
                                  };
                            }
                            throw internalError;
                          }
                        } else {
                          newArray.push(innerData);
                        }
                      }
                      return newArray;
                    }));
            }),
          r: 0,
          e: 0,
          s: initialSerialize,
          p: intitialParse,
          a: intitialParseAsync,
          i: undefined,
          m: undefined
        };
}

var HackyValidValue = /* @__PURE__ */Caml_exceptions.create("S-ReScriptStruct.Union.HackyValidValue");

function factory$17(structs) {
  if (structs.length < 2) {
    throw new Error("[rescript-struct] A Union struct factory require at least two structs.");
  }
  return {
          n: "Union",
          t: {
            TAG: /* Union */6,
            _0: structs
          },
          pf: (function (ctx, compilingStruct) {
              var structs = compilingStruct.t._0;
              var noopOps = [];
              var syncOps = [];
              var asyncOps = [];
              for(var idx = 0 ,idx_finish = structs.length; idx < idx_finish; ++idx){
                var innerStruct = structs[idx];
                var fn = getParseOperation(innerStruct);
                if (typeof fn === "number") {
                  noopOps.push(undefined);
                } else if (fn.TAG === /* SyncOperation */0) {
                  syncOps.push([
                        idx,
                        fn._0
                      ]);
                } else {
                  asyncOps.push([
                        idx,
                        fn._0
                      ]);
                }
              }
              var withAsyncOps = asyncOps.length > 0;
              if (noopOps.length === 0) {
                planSyncTransformation(ctx, (function (input) {
                        var idxRef = 0;
                        var errorsRef = [];
                        var maybeNewValueRef;
                        while(idxRef < syncOps.length && maybeNewValueRef === undefined) {
                          var idx = idxRef;
                          var match = syncOps[idx];
                          try {
                            var newValue = match[1](input);
                            maybeNewValueRef = Caml_option.some(newValue);
                          }
                          catch (raw_internalError){
                            var internalError = Caml_js_exceptions.internalToOCamlException(raw_internalError);
                            if (internalError.RE_EXN_ID === Exception) {
                              errorsRef[match[0]] = internalError._1;
                              idxRef = idxRef + 1;
                            } else {
                              throw internalError;
                            }
                          }
                        }                        var match$1 = maybeNewValueRef;
                        if (match$1 !== undefined) {
                          if (withAsyncOps) {
                            return {
                                    maybeSyncValue: match$1,
                                    tempErrors: errorsRef,
                                    originalInput: input
                                  };
                          } else {
                            return Caml_option.valFromOption(match$1);
                          }
                        } else if (withAsyncOps) {
                          return {
                                  maybeSyncValue: match$1,
                                  tempErrors: errorsRef,
                                  originalInput: input
                                };
                        } else {
                          return raise$1({
                                      TAG: /* InvalidUnion */5,
                                      _0: errorsRef.map(toParseError)
                                    });
                        }
                      }));
                if (withAsyncOps) {
                  return planAsyncTransformation(ctx, (function (input) {
                                var syncValue = input.maybeSyncValue;
                                if (syncValue !== undefined) {
                                  return Promise.resolve(Caml_option.valFromOption(syncValue));
                                } else {
                                  return Promise.all(asyncOps.map(function (param) {
                                                    var originalIdx = param[0];
                                                    try {
                                                      return param[1](input.originalInput)().then((function (value) {
                                                                    throw {
                                                                          RE_EXN_ID: HackyValidValue,
                                                                          _1: value,
                                                                          Error: new Error()
                                                                        };
                                                                  }), (function (exn) {
                                                                    if (exn.RE_EXN_ID === Exception) {
                                                                      var array = input.tempErrors;
                                                                      array[originalIdx] = exn._1;
                                                                      return ;
                                                                    }
                                                                    throw exn;
                                                                  }));
                                                    }
                                                    catch (raw_internalError){
                                                      var internalError = Caml_js_exceptions.internalToOCamlException(raw_internalError);
                                                      if (internalError.RE_EXN_ID === Exception) {
                                                        var array = input.tempErrors;
                                                        return Promise.resolve((array[originalIdx] = internalError._1, undefined));
                                                      }
                                                      throw internalError;
                                                    }
                                                  })).then((function (param) {
                                                return raise$1({
                                                            TAG: /* InvalidUnion */5,
                                                            _0: input.tempErrors.map(toParseError)
                                                          });
                                              }), (function (exn) {
                                                if (exn.RE_EXN_ID === HackyValidValue) {
                                                  return exn._1;
                                                }
                                                throw exn;
                                              }));
                                }
                              }));
                } else {
                  return ;
                }
              }
              
            }),
          sf: (function (ctx, param) {
              var serializeOperations = [];
              for(var idx = 0 ,idx_finish = structs.length; idx < idx_finish; ++idx){
                serializeOperations.push(getSerializeOperation(structs[idx]));
              }
              planSyncTransformation(ctx, (function (input) {
                      var idxRef = 0;
                      var errors = [];
                      var maybeNewValueRef;
                      while(idxRef < serializeOperations.length && maybeNewValueRef === undefined) {
                        var idx = idxRef;
                        var serializeOperation = serializeOperations[idx];
                        try {
                          var newValue = serializeOperation !== undefined ? serializeOperation(input) : input;
                          maybeNewValueRef = Caml_option.some(newValue);
                        }
                        catch (raw_internalError){
                          var internalError = Caml_js_exceptions.internalToOCamlException(raw_internalError);
                          if (internalError.RE_EXN_ID === Exception) {
                            errors.push(internalError._1);
                            idxRef = idxRef + 1;
                          } else {
                            throw internalError;
                          }
                        }
                      }                      var ok = maybeNewValueRef;
                      if (ok !== undefined) {
                        return Caml_option.valFromOption(ok);
                      } else {
                        return raise$1({
                                    TAG: /* InvalidUnion */5,
                                    _0: errors.map(toSerializeError)
                                  });
                      }
                    }));
            }),
          r: 0,
          e: 0,
          s: initialSerialize,
          p: intitialParse,
          a: intitialParseAsync,
          i: undefined,
          m: undefined
        };
}

function getExn(result) {
  if (result.TAG === /* Ok */0) {
    return result._0;
  }
  var message = toString(result._0);
  throw new Error("[rescript-struct] " + message + "");
}

function mapErrorToString(result) {
  if (result.TAG === /* Ok */0) {
    return result;
  } else {
    return {
            TAG: /* Error */1,
            _0: toString(result._0)
          };
  }
}

var Result = {
  getExn: getExn,
  mapErrorToString: mapErrorToString
};

var string = factory$5;

var literal = factory$1;

var literalVariant = factory;

var array = factory$13;

var option = factory$11;

var json = factory$6;

var union = factory$17;

var defaulted = factory$15;

var Object_UnknownKeys = {
  classify: classify$1
};

var $$Object = {
  UnknownKeys: Object_UnknownKeys,
  strip: strip,
  strict: strict
};

var object = factory$2;

var tuple0 = factory$16;

var tuple1 = factory$16;

var tuple2 = factory$16;
/* value Not a pure module */

// Generated by ReScript, PLEASE EDIT WITH CARE
function getGlobalyOpenedModules(bsConfig) {
  var globalyOpenedModules = [];
  bsConfig.bscFlags.forEach(function (bscFlag) {
    forEach(fromBscFlag(bscFlag), function (moduleName) {
      globalyOpenedModules.push(moduleName);
    });
  });
  return globalyOpenedModules;
}
function lint(bsConfig, prohibitedModuleNames) {
  var globalyOpenedModules = getGlobalyOpenedModules(bsConfig);
  var maybeOpenedProhibitedModule = prohibitedModuleNames.find(function (prohibitedModule) {
    return globalyOpenedModules.some(function (globalyOpenedModule) {
      return globalyOpenedModule === prohibitedModule;
    });
  });
  if (maybeOpenedProhibitedModule !== undefined) {
    return {
      TAG: /* Error */1,
      _0: {
        NAME: "HAS_OPENED_PROHIBITED_MODULE",
        VAL: maybeOpenedProhibitedModule
      }
    };
  } else {
    return {
      TAG: /* Ok */0,
      _0: undefined
    };
  }
}
var struct$1 = json(object(function (o) {
  return {
    bscFlags: field(o, "bsc-flags", defaulted(option(array(string())), []))
  };
}));
function fromJsonString$1(jsonString) {
  return Result.mapErrorToString(parseWith(jsonString, struct$1));
}
/* struct Not a pure module */

// Generated by ReScript, PLEASE EDIT WITH CARE
function getProjectDirs(projectDirs) {
  return projectDirs;
}
var struct = json(object(function (o) {
  return field(o, "dirs", array(string()));
}));
function fromJsonString(jsonString) {
  return Result.mapErrorToString(parseWith(jsonString, struct));
}
/* struct Not a pure module */

// Generated by ReScript, PLEASE EDIT WITH CARE
function mapError(result, fn) {
  if (result.TAG === /* Ok */0) {
    return result;
  } else {
    return {
      TAG: /* Error */1,
      _0: fn(result._0)
    };
  }
}
var flatMap = Belt_Result.flatMapU;
var map = Belt_Result.mapU;
/* No side effect */

// Generated by ReScript, PLEASE EDIT WITH CARE
function make$6(projectPath, loadBsConfig, loadSourceDirs) {
  return function () {
    return flatMap(flatMap(flatMap(mapError(loadBsConfig(), function (loadBsConfigError) {
      return {
        TAG: /* BsConfigParseFailure */0,
        _0: loadBsConfigError._0
      };
    }), function (bsConfig) {
      return mapError(lint(bsConfig, defaultProhibitedModuleNames), function (error) {
        return {
          TAG: /* BsConfigHasOpenedProhibitedModule */2,
          _0: error.VAL
        };
      });
    }), function (param) {
      return mapError(loadSourceDirs(), function (loadSourceDirsError) {
        if (loadSourceDirsError) {
          return {
            TAG: /* SourceDirsParseFailure */1,
            _0: loadSourceDirsError._0
          };
        } else {
          return (/* RescriptCompilerArtifactsNotFound */0
          );
        }
      });
    }), function (sourceDirs) {
      var resFiles = getProjectDirs(sourceDirs).flatMap(function (sourceDir) {
        var fullDirPath = Path.resolve(projectPath, sourceDir);
        return Fs.readdirSync(fullDirPath).filter(checkIsResFile).map(function (dirItem) {
          var resFilePath = "" + fullDirPath + "/" + dirItem + "";
          return make$7(Fs.readFileSync(resFilePath, {
            encoding: "utf8"
          }).toString(), resFilePath);
        });
      });
      var lintContext = make$8();
      resFiles.forEach(function (resFile) {
        lint$1(resFile, lintContext, defaultProhibitedModuleNames, defaultStdlibModuleName);
      });
      var lintIssues = getIssues(lintContext);
      if (lintIssues.length !== 0) {
        return {
          TAG: /* Error */1,
          _0: {
            TAG: /* LintFailedWithIssues */3,
            _0: lintIssues
          }
        };
      } else {
        return {
          TAG: /* Ok */0,
          _0: undefined
        };
      }
    });
  };
}
/* fs Not a pure module */

// Generated by ReScript, PLEASE EDIT WITH CARE
function make$5(runLintCommand, runHelpCommand, runHelpLintCommand) {
  return function () {
    var commandArguments = Process.argv.slice(2);
    var result = map(mapError(parseWith(Minimist(commandArguments), union([$$Object.strict(object(function (o) {
      discriminant(o, "_", union([tuple0(), tuple1(literalVariant({
        TAG: /* String */0,
        _0: "help"
      }, undefined))]));
      return (/* Help */0
      );
    })), $$Object.strict(object(function (o) {
      discriminant(o, "_", tuple2(literal({
        TAG: /* String */0,
        _0: "help"
      }), literal({
        TAG: /* String */0,
        _0: "lint"
      })));
      return (/* LintHelp */2
      );
    })), $$Object.strict(object(function (o) {
      discriminant(o, "_", tuple1(literal({
        TAG: /* String */0,
        _0: "lint"
      })));
      return (/* Lint */1
      );
    }))])), function (error) {
      var unionErrors = error.code;
      if (typeof unionErrors === "number") {
        return Js_exn.raiseError("Parsed error always must have the InvalidUnion code");
      }
      if (unionErrors.TAG !== /* InvalidUnion */5) {
        return Js_exn.raiseError("Parsed error always must have the InvalidUnion code");
      }
      var maybeIllegalOptionName = map$1(Caml_option.undefined_to_opt(unionErrors._0.find(function (error) {
        var match = error.code;
        if (typeof match === "number" || match.TAG !== /* ExcessField */4) {
          return false;
        } else {
          return true;
        }
      })), function (excessFieldError) {
        var illegalOptionName = excessFieldError.code;
        if (typeof illegalOptionName === "number" || illegalOptionName.TAG !== /* ExcessField */4) {
          return Js_exn.raiseError("The excessFieldError always must have the ExcessField code");
        } else {
          return illegalOptionName._0;
        }
      });
      if (maybeIllegalOptionName !== undefined) {
        return (/* IllegalOption */{
            optionName: maybeIllegalOptionName
          }
        );
      } else {
        return (/* CommandNotFound */0
        );
      }
    }), function (command) {
      switch (command) {
        case /* Help */0:
          return runHelpCommand();
        case /* Lint */1:
          return runLintCommand();
        case /* LintHelp */2:
          return runHelpLintCommand();
      }
    });
    if (result.TAG === /* Ok */0) {
      return;
    }
    var error = result._0;
    if (error) {
      console.log("Illegal option:", error.optionName);
    } else {
      console.log("Command not found:", commandArguments.join(" "));
    }
    Process.exit(1);
  };
}
/* process Not a pure module */

// Generated by ReScript, PLEASE EDIT WITH CARE
function make$4(projectPath) {
  return function () {
    return mapError(fromJsonString$1(Fs.readFileSync(Path.resolve(projectPath, "bsconfig.json"), {
      encoding: "utf8"
    }).toString()), function (error) {
      return (/* ParsingFailure */{
          _0: error
        }
      );
    });
  };
}
/* fs Not a pure module */

// Generated by ReScript, PLEASE EDIT WITH CARE
function make$3(projectPath) {
  return function () {
    var tmp;
    try {
      tmp = {
        TAG: /* Ok */0,
        _0: Fs.readFileSync(Path.resolve(projectPath, "lib/bs/.sourcedirs.json"), {
          encoding: "utf8"
        }).toString()
      };
    } catch (exn) {
      tmp = {
        TAG: /* Error */1,
        _0: /* RescriptCompilerArtifactsNotFound */0
      };
    }
    return flatMap(tmp, function (file) {
      return mapError(fromJsonString(file), function (error) {
        return (/* ParsingFailure */{
            _0: error
          }
        );
      });
    });
  };
}
/* fs Not a pure module */

// Generated by ReScript, PLEASE EDIT WITH CARE
function make$2(param) {
  return function () {
    console.log(["", "Available commands are:", "", "* " + Colorette.bold("help") + " - Display this message", "* " + Colorette.bold("help <command>") + " - Show more information about a command", "* " + Colorette.bold("lint") + " - Lint rescript standard libriries usage", "", "You can find more information about the reasoning behind the tool in the documentation: " + Colorette.underline("https://github.com/DZakh/rescript-stdlib-vendorer") + "", ""].join("\n"));
  };
}
/* colorette Not a pure module */

// Generated by ReScript, PLEASE EDIT WITH CARE
function make$1(lint) {
  return function () {
    var error = lint();
    if (error.TAG === /* Ok */0) {
      return;
    }
    var error$1 = error._0;
    if (typeof error$1 === "number") {
      console.log("Couldn't find rescript compiler artifacts in the \"./lib/bs/\" directory. Try to run compiler before the lint script.");
    } else {
      switch (error$1.TAG | 0) {
        case /* BsConfigParseFailure */0:
          console.log("Failed to parse \"bsconfig.json\":", error$1._0);
          break;
        case /* SourceDirsParseFailure */1:
          console.log("Failed to parse \".sourcedirs.json\". Check that you use compatible ReScript version. Parsing error:", error$1._0);
          break;
        case /* BsConfigHasOpenedProhibitedModule */2:
          console.log("Lint failed: Found globally opened module " + error$1._0 + "");
          break;
        case /* LintFailedWithIssues */3:
          error$1._0.forEach(function (lintIssue) {
            console.log(Colorette.underline(getLink(lintIssue)), "\n", getMessage(lintIssue), "\n");
          });
          console.log(Colorette.bold("Use custom standard library. Read more in the documentation: " + Colorette.underline("https://github.com/DZakh/rescript-stdlib-vendorer") + ""));
          break;
      }
    }
    Process.exit(1);
  };
}
/* process Not a pure module */

// Generated by ReScript, PLEASE EDIT WITH CARE

function make(param) {
  return function () {
    console.log(["", "Lint rescript standard libriries usage.", ""].join("\n"));
  };
}
/* No side effect */

// Generated by ReScript, PLEASE EDIT WITH CARE
var projectPath = Process.cwd();
var runCli = make$5(make$1(make$6(projectPath, make$4(projectPath), make$3(projectPath))), make$2(), make());
runCli();
/* projectPath Not a pure module */
//# sourceMappingURL=rescript-stdlib-vendorer.modern.js.map
